#include <Arduino.h>
#include <functional>
#include "WifiControl.h"
#include "BrowserGui.h"
#include "DnsControl.h"
#include "LedControl.h"
#include "Timecontrol.h"
#include "CallbackFunction.h"
#include "UpnpBroadcastResponder.h"
#include "Device.h"
#include "AmbientLightControl.h"
#include "Storage.h"
#include <FastLED.h>

void ClockOn();
void ClockOff();

static TimeControl& timecontrol = TimeControl::getInstance();
static WifiControl& wificontrol = WifiControl::getInstance();
static BrowserGui& browsergui = BrowserGui::getInstance();
static DnsControl& dnscontrol = DnsControl::getInstance();
static LedControl& ledcontrol = LedControl::getInstance();
static AmbientLightControl& ambcontrol = AmbientLightControl::getInstance();
UpnpBroadcastResponder *upnpBroadcastResponder;
Device *Clock = NULL;

/* indicator, if one time has passed, then update leds */
uint8_t last_minutes = 0;
uint8_t last_hours = 0;
uint8_t last_seconds = 0;

void setup()
{
  Serial.begin(115200);
  Serial.println("");

  storageInit();
  ambcontrol.start();
  ledcontrol.start();
  wificontrol.connectWifi();
  browsergui.start();
  timecontrol.start();
  dnscontrol.start();
  /*ALEXA control*/
  Clock = new Device("Uhr", 81, ClockOn, ClockOff);
  upnpBroadcastResponder = new UpnpBroadcastResponder();
  upnpBroadcastResponder->addDevice(*Clock);
  upnpBroadcastResponder->start();
}

void loop()
{
  uint8_t hours = 0;
  uint8_t minutes = 0;
  uint8_t seconds = 0;

  browsergui.serverLoop();
  dnscontrol.serverLoop();
  upnpBroadcastResponder->serverLoop();
  Clock->serverLoop();

  if (wificontrol.isWifiConnected())
  {
    timecontrol.getTime(&hours, &minutes, &seconds);
        if (last_minutes != minutes || last_hours != hours)
    {
      Serial.print("Time: ");
      Serial.print(hours);
      Serial.print(":");
      Serial.println(minutes);
      last_minutes = minutes;
      last_hours = hours;
      ledcontrol.setTime(hours, minutes);
    }
    if(last_seconds != seconds)
    {
      last_seconds = seconds;
      ambcontrol.updateBrightness();
    }
  }
}

void ClockOn()
{
  ledcontrol.unsetTemporaryColor();
}

void ClockOff()
{
  CRGB newColor = 0;
  ledcontrol.setTemporaryColor(newColor);
}
