#include "Device.h"
#include "CallbackFunction.h"

Device::Device()
{
}

Device::Device(String alexaInvokeName, unsigned int port, CallbackFunction oncb, CallbackFunction offcb) {
  uint32_t chipId = ESP.getChipId();
  char uuid[64];
  sprintf_P(uuid, PSTR("38323636-4558-4dda-9188-cda0e6%02x%02x%02x"),
            (uint16_t) ((chipId >> 16) & 0xff),
            (uint16_t) ((chipId >>  8) & 0xff),
            (uint16_t)   chipId        & 0xff);

  serial = String(uuid);
  persistent_uuid = "Socket-1_0-" + serial + "-" + String(port);

  device_name = alexaInvokeName;
  localPort = port;
  onCallback = oncb;
  offCallback = offcb;

  startWebServer();
}

void Device::serverLoop() {
  if (server != NULL) {
    server->handleClient();
    delay(1);
  }
}

void Device::startWebServer() {
  server = new ESP8266WebServer(localPort);

  server->on("/setup.xml", [&]() {
    handleSetupXml();
  });

  server->on("/upnp/control/basicevent1", [&]() {
    handleUpnpControl();
  });

  server->begin();

  Serial.println("Alexa service for " + device_name + " started.");
}

void Device::handleUpnpControl() {
  Serial.print(device_name);
  Serial.println(" responding to Alexa order");

  String request = server->arg(0);

  if (request.indexOf("<BinaryState>1</BinaryState>") > 0) {
    onCallback();
  }

  if (request.indexOf("<BinaryState>0</BinaryState>") > 0) {
    offCallback();
  }
  server->send(200, "text/plain", "");
}

void Device::handleSetupXml() {
  Serial.print(device_name);
  Serial.println(" responding request for setup.xml");

  String setup_xml = "<?xml version=\"1.0\"?>"
                     "<root>"
                     "<device>"
                     "<deviceType>urn:Belkin:device:controllee:1</deviceType>"
                     "<friendlyName>" + device_name + "</friendlyName>"
                     "<manufacturer>Belkin International Inc.</manufacturer>"
                     "<modelName>Emulated Socket</modelName>"
                     "<modelNumber>1.0.0</modelNumber>"
                     "<UDN>uuid:" + persistent_uuid + "</UDN>"
                     "</device>"
                     "</root>\r\n"
                     "\r\n";

  server->send(200, "text/xml", setup_xml.c_str());
}

String Device::getAlexaInvokeName() {
  return device_name;
}

void Device::respondToBelkinSearch(IPAddress& senderIP, unsigned int senderPort) {
  Serial.print(device_name);
  Serial.print(" responding to search from IP:");
  Serial.print(senderIP);
  Serial.print(" Port: ");
  Serial.println(senderPort);

  String response =
    "HTTP/1.1 200 OK\r\n"
    "CACHE-CONTROL: max-age=86400\r\n"
    "DATE: Sat, 04 Feb 2017 04:56:29 GMT\r\n"
    "EXT:\r\n"
    "LOCATION: http://" + WiFi.localIP().toString() + ":" + String(localPort) + "/setup.xml\r\n"
    "OPT: \"http://schemas.upnp.org/upnp/1/0/\"; ns=01\r\n"
    "01-NLS: b9200ebb-736d-4b93-bf03-835149d13983\r\n"
    "SERVER: Unspecified, UPnP/1.0, Unspecified\r\n"
    "ST: urn:Belkin:device:**\r\n"
    "USN: uuid:" + persistent_uuid + "::urn:Belkin:device:**\r\n\r\n";

  UDP.beginPacket(senderIP, senderPort);
  UDP.write(response.c_str());
  UDP.endPacket();
}

void Device::switchOnDevice() {
  onCallback();
}
void Device::switchOffDevice() {
  offCallback();
}
