#include "LedControl.h"
#include "LEDMatrix.h"
#include "Storage.h"

/* 110 LEDs for display. First LED is used as level shifter and has to be black */
#define NUM_LEDS 111
#define DATA_PIN 2
#define on showColor
#define off CRGB::Black

CRGB chain[NUM_LEDS];

LedControl& LedControl::getInstance() {
  static LedControl instance;
  return instance;
}

LedControl::LedControl()
{
}

void LedControl::start()
{
  FastLED.addLeds<WS2812B, DATA_PIN, GRB>(chain, NUM_LEDS);
  loadColor(&showColor.red, &showColor.green, &showColor.blue);
  loadNightBegin(&nightBegin);
  loadNightEnd(&nightEnd);
  loadDreiviertelMode(&dreiviertelMode);
  hh = 0;
  mm = 0;

  clearLeds();
  FastLED.show();
}

void LedControl::setColor(CRGB color)
{
  showColor = color;
  saveColor(color.red, color.green, color.blue);
  updateLeds();
}
void LedControl::setColor(String string)
{
  String color_string = "";
  long int color_r;
  CRGB newColor;

  string.replace("#", "0x");
  color_r = strtol(string.c_str(), NULL, 0);
  newColor = (CRGB)color_r;
  setColor(newColor);
}

void LedControl::setTemporaryColor(CRGB color)
{
  showColor = color;
  updateLeds();
}
void LedControl::unsetTemporaryColor()
{
  loadColor(&showColor.red, &showColor.green, &showColor.blue);
  updateLeds();
}

void LedControl::setBrightness(uint8_t brightness)
{
  uint8_t newBrightness = map(brightness, 0, 100, 0, 255);
  uint8_t brightnessStep = 1;
  if (showBrightness > newBrightness)
  {
    brightnessStep = -1;
  }
  else
  {
    brightnessStep = 1;
  }
  while (showBrightness != newBrightness)
  {
    showBrightness += brightnessStep;
    updateLeds();
  }
}

void LedControl::setTime(uint8_t hours, uint8_t minutes)
{
  hh = hours;
  mm = minutes;
  updateLeds();
}

void LedControl::clearLeds(void)
{
  for (int i = 0; i < NUM_LEDS; i++)
  {
    chain[i] = off;
  }
}

void LedControl::updateLeds()
{
  clearLeds();
  if (!isNightmode())
  {
    TimeToDisplay();
    FastLED.setBrightness(showBrightness);
  }
  else 
  {
    FastLED.show();
  }
  FastLED.delay(1000 / 250);
}

void LedControl::setNightmode(String string1, String string2)
{
  String tmpStr = "";
  uint8_t nightBeginHours = 0;
  uint8_t nightBeginMinutes = 0;
  uint8_t nightEndHours = 0;
  uint8_t nightEndMinutes = 0;

  tmpStr = string1.substring(0, 2);
  nightBeginHours = (uint8_t)tmpStr.toInt();
  tmpStr = string1.substring(3, 5);
  nightBeginMinutes = (uint8_t)tmpStr.toInt();

  tmpStr = string2.substring(0, 2);
  nightEndHours = (uint8_t)tmpStr.toInt();
  tmpStr = string2.substring(3, 5);
  nightEndMinutes = (uint8_t)tmpStr.toInt();

  setNightBegin(nightBeginHours, nightBeginMinutes);
  setNightEnd(nightEndHours, nightEndMinutes);
}

void LedControl::setNightBegin(uint8_t hours, uint8_t minutes)
{
  nightBegin = ((uint32_t)hours * (uint32_t)60) + (uint32_t)minutes;
  saveNightBegin(nightBegin);
}

void LedControl::setNightEnd(uint8_t hours, uint8_t minutes)
{
  nightEnd = ((uint32_t)hours * (uint32_t)60) + (uint32_t)minutes;
  saveNightEnd(nightEnd);
  updateLeds();
}

void LedControl::TimeToDisplay()
{
  DISPLAY_ESIST

  if (mm >= 3 && mm <= 7) {
    DISPLAY_FUENF_2
    DISPLAY_NACH
  }
  else if (mm >= 8 && mm <= 12) {
    DISPLAY_ZEHN_2
    DISPLAY_NACH
  }
  else if (mm >= 13 && mm <= 17) {
    DISPLAY_VIERTEL
    DISPLAY_NACH
  }
  else if (mm >= 18 && mm <= 22) {
    DISPLAY_ZWANZIG
    DISPLAY_NACH
  }
  else if (mm >= 23 && mm <= 27) {
    DISPLAY_FUENF_2
    DISPLAY_VOR
    DISPLAY_HALB
  }
  else if (mm >= 28 && mm <= 32) {
    DISPLAY_HALB
  }
  else if (mm >= 33 && mm <= 37) {
    DISPLAY_FUENF_2
    DISPLAY_NACH
    DISPLAY_HALB
  }
  else if (mm >= 38 && mm <= 42) {
    DISPLAY_ZWANZIG
    DISPLAY_VOR
  }
  else if (mm >= 43 && mm <= 47) {
    if (dreiviertelMode)
    {
      DISPLAY_DREIVIERTEL
    }
    else
    {
      DISPLAY_VIERTEL
      DISPLAY_VOR
    }
  }
  else if (mm >= 48 && mm <= 52) {
    DISPLAY_ZEHN_2
    DISPLAY_VOR
  }
  else if (mm >= 53 && mm <= 57) {
    DISPLAY_FUENF_2
    DISPLAY_VOR
  }
  else {
    //full hour
  }

  if ( (hh == 0 && mm >= 58) || (hh == 1 && mm <= 2) || (hh == 12 && mm >= 58) || (hh == 13 && mm <= 2)) {
    DISPLAY_EIN
  }
  else if ( (hh == 0 && mm >= 23) || (hh == 1 && mm <= 22) || (hh == 12 && mm >= 23) || (hh == 13 && mm <= 22) ) {
    DISPLAY_EINS
  }
  else if ( (hh == 1 && mm >= 23) || (hh == 2 && mm <= 22) || (hh == 13 && mm >= 23) || (hh == 14 && mm <= 22)) {
    DISPLAY_ZWEI
  }
  else if ( (hh == 2 && mm >= 23) || (hh == 3 && mm <= 22) || (hh == 14 && mm >= 23) || (hh == 15 && mm <= 22)) {
    DISPLAY_DREI
  }
  else if ( (hh == 3 && mm >= 23) || (hh == 4 && mm <= 22) || (hh == 15 && mm >= 23) || (hh == 16 && mm <= 22)) {
    DISPLAY_VIER
  }
  else if ( (hh == 4 && mm >= 23) || (hh == 5 && mm <= 22) || (hh == 16 && mm >= 23) || (hh == 17 && mm <= 22)) {
    DISPLAY_FUENF
  }
  else if ( (hh == 5 && mm >= 23) || (hh == 6 && mm <= 22) || (hh == 17 && mm >= 23) || (hh == 18 && mm <= 22)) {
    DISPLAY_SECHS
  }
  else if ( (hh == 6 && mm >= 23) || (hh == 7 && mm <= 22) || (hh == 18 && mm >= 23) || (hh == 19 && mm <= 22)) {
    DISPLAY_SIEBEN
  }
  else if ( (hh == 7 && mm >= 23) || (hh == 8 && mm <= 22) || (hh == 19 && mm >= 23) || (hh == 20 && mm <= 22)) {
    DISPLAY_ACHT
  }
  else if ( (hh == 8 && mm >= 23) || (hh == 9 && mm <= 22) || (hh == 20 && mm >= 23) || (hh == 21 && mm <= 22)) {
    DISPLAY_NEUN
  }
  else if ( (hh == 9 && mm >= 23) || (hh == 10 && mm <= 22) || (hh == 21 && mm >= 23) || (hh == 22 && mm <= 22)) {
    DISPLAY_ZEHN
  }
  else if ( (hh == 10 && mm >= 23) || (hh == 11 && mm <= 22) || (hh == 22 && mm >= 23) || (hh == 23 && mm <= 22)) {
    DISPLAY_ELF
  }
  else if ( (hh == 23 && mm >= 23) || (hh == 0 && mm <= 22) || (hh == 11 && mm >= 23) || (hh == 12 && mm <= 22)) {
    DISPLAY_ZWOELF
  }
  else {
    /*nothing*/
  }
  if (mm >= 58 || mm <= 2) {
    DISPLAY_UHR
  }
}


bool LedControl::isNightmode()
{
  bool isNight = false;
  uint32_t currentTime = ((uint32_t)hh * (uint32_t)60) + (uint32_t)mm;

  if (((nightBegin < nightEnd) && (currentTime >= nightBegin) && (currentTime < nightEnd))
      || ((nightBegin > nightEnd) && ((currentTime >= nightBegin) || (currentTime < nightEnd))))
  {
    isNight = true;
  }
  return isNight;
}

void LedControl::setDreiviertelMode(bool newMode)
{
  dreiviertelMode = newMode;
  updateLeds();
  saveDreiviertelMode(dreiviertelMode);
}

bool LedControl::getDreiviertelMode()
{
  return dreiviertelMode;
}

