#include "Storage.h"
#include <EEPROM.h>

int FLASHPATTERN = 3735931646;//0xDEADCAFE == 3735931646

void storageInit()
{
  EEPROM.begin(EEPROM_SIZE);
  checkFirstFlash();
}

void checkFirstFlash()
{
  int readpattern = 0;
  readpattern += EEPROM.read(FIRSTFLASH_START);
  readpattern += (EEPROM.read(FIRSTFLASH_START + 1) << 8);
  readpattern += (EEPROM.read(FIRSTFLASH_START + 2) << 16);
  readpattern += (EEPROM.read(FIRSTFLASH_START + 3) << 24);

  if (FLASHPATTERN != readpattern)
  {
    intitializeDefaultValues();
  }
}

void saveWifiData(char* ssid, char* password)
{
  Serial.println("Saving wifi data");
  for (int i = 0; i < WIFI_DATA_LENGTH; i++)
  {
    EEPROM.write(SSID_START + i, ssid[i]);
  }

  for (int i = 0; i < WIFI_DATA_LENGTH; i++)
  {
    EEPROM.write(PASSWORD_START + i, password[i]);
  }

  EEPROM.commit();
}

void loadWifiData(char *ssid, char* password)
{
  for (int i = 0; i < WIFI_DATA_LENGTH; i++)
  {
    ssid[i] = EEPROM.read(SSID_START + i);
  }

  for (int i = 0; i < WIFI_DATA_LENGTH; i++)
  {
    password[i] = EEPROM.read(PASSWORD_START + i);
  }

  Serial.println("Loaded wifi data");
}

void saveColor(uint8_t r, uint8_t b, uint8_t g)
{
  Serial.print("Saving color: ");
  Serial.print(r);
  Serial.print(", ");
  Serial.print(b);
  Serial.print(", ");
  Serial.println(g);

  EEPROM.write(COLOR_START, r);
  EEPROM.write(COLOR_START + 1, g);
  EEPROM.write(COLOR_START + 2, b);
  EEPROM.commit();
}

void loadColor(uint8_t *r, uint8_t *b, uint8_t *g)
{
  *r = EEPROM.read(COLOR_START);
  *g = EEPROM.read(COLOR_START + 1);
  *b = EEPROM.read(COLOR_START + 2);

  Serial.print("Loaded color: ");
  Serial.print(*r);
  Serial.print(", ");
  Serial.print(*b);
  Serial.print(", ");
  Serial.println(*g);
}

void saveMinBrightness(uint8_t brightness)
{
  Serial.print("Saving min brightness: ");
  Serial.println(brightness);

  EEPROM.write(BRIGHTNESS_START, brightness);
  EEPROM.commit();
}

void loadMinBrightness(uint8_t *brightness)
{
  *brightness = EEPROM.read(BRIGHTNESS_START);

  Serial.print("loaded min brightness: ");
  Serial.println(*brightness);
}

void saveBrightnessAuto(bool newValue)
{
  if (newValue)
  {
    Serial.println("Saving brightnessauto: on");
    EEPROM.write(BRIGHTNESSAUTO_START, 0xFF);
  }
  else
  {
    Serial.println("Saving brightnessauto: off");
    EEPROM.write(BRIGHTNESSAUTO_START, 0x00);
  }
  EEPROM.commit();
}
void loadBrightnessAuto(bool *value)
{
  char tmp = EEPROM.read(BRIGHTNESSAUTO_START);
  if (0xFF == tmp)
  {
    *value = true;
    Serial.println("loaded brightnessauto: on");
  }
  else
  {
    *value = false;
    Serial.println("loaded brightnessauto: on");
  }
}


void saveNightBegin(uint32_t nightBegin)
{
  EEPROM.write(NIGHTBEGIN_START, (uint8_t)(nightBegin));
  EEPROM.write((NIGHTBEGIN_START + 1), (uint8_t)(nightBegin >> 8));
  EEPROM.write((NIGHTBEGIN_START + 2), (uint8_t)(nightBegin >> 16));
  EEPROM.write((NIGHTBEGIN_START + 3), (uint8_t)(nightBegin >> 24));
  EEPROM.commit();
}

void loadNightBegin(uint32_t *nightBegin)
{
  int tmp = 0;
  tmp += EEPROM.read(NIGHTBEGIN_START);
  tmp += (EEPROM.read(NIGHTBEGIN_START + 1) << 8);
  tmp += (EEPROM.read(NIGHTBEGIN_START + 2) << 16);
  tmp += (EEPROM.read(NIGHTBEGIN_START + 3) << 24);
  *nightBegin = tmp;
}
void saveNightEnd(uint32_t nightEnd)
{
  EEPROM.write((NIGHTEND_START), (uint8_t)(nightEnd));
  EEPROM.write((NIGHTEND_START + 1), (uint8_t)(nightEnd >> 8));
  EEPROM.write((NIGHTEND_START + 2), (uint8_t)(nightEnd >> 16));
  EEPROM.write((NIGHTEND_START + 3), (uint8_t)(nightEnd >> 24));
  EEPROM.commit();
}
void loadNightEnd(uint32_t *nightEnd)
{
  int tmp = 0;
  tmp += EEPROM.read(NIGHTEND_START);
  tmp += (EEPROM.read(NIGHTEND_START + 1) << 8);
  tmp += (EEPROM.read(NIGHTEND_START + 2) << 16);
  tmp += (EEPROM.read(NIGHTEND_START + 3) << 24);
  *nightEnd = tmp;
}

void saveUtcDiff(int utcDiff)
{
  EEPROM.write((UTC_DIFF_START), (uint8_t)(utcDiff));
  EEPROM.write((UTC_DIFF_START + 1), (uint8_t)(utcDiff >> 8));
  EEPROM.write((UTC_DIFF_START + 2), (uint8_t)(utcDiff >> 16));
  EEPROM.write((UTC_DIFF_START + 3), (uint8_t)(utcDiff >> 24));
  EEPROM.commit();
}

void loadUtcDiff(int *utcDiff)
{
  int tmp = 0;
  tmp += EEPROM.read(UTC_DIFF_START);
  tmp += (EEPROM.read(UTC_DIFF_START + 1) << 8);
  tmp += (EEPROM.read(UTC_DIFF_START + 2) << 16);
  tmp += (EEPROM.read(UTC_DIFF_START + 3) << 24);
  *utcDiff = tmp;
}

void saveSummertimeAuto(bool newValue)
{
  if (newValue)
  {
    Serial.println("Saving summertimeauto: on");
    EEPROM.write(SUMMERTIMEAUTO_START, 0xFF);
  }
  else
  {
    Serial.println("Saving summertimeauto: off");
    EEPROM.write(SUMMERTIMEAUTO_START, 0x00);
  }
  EEPROM.commit();
}
void loadSummertimeAuto(bool *value)
{
  char tmp = EEPROM.read(SUMMERTIMEAUTO_START);
  if (0xFF == tmp)
  {
    *value = true;
    Serial.println("loaded summertimeauto: on");
  }
  else
  {
    *value = false;
    Serial.println("loaded summertimeauto: off");
  }
}

void saveDreiviertelMode(bool newValue)
{
  if (newValue)
  {
    Serial.println("Saving dreiviertel");
    EEPROM.write(DREIVIERTELMODE_START, 0xFF);
  }
  else
  {
    Serial.println("Saving viertel vor");
    EEPROM.write(DREIVIERTELMODE_START, 0x00);
  }
  EEPROM.commit();
}
void loadDreiviertelMode(bool *value)
{
  char tmp = EEPROM.read(DREIVIERTELMODE_START);
  if (0xFF == tmp)
  {
    *value = true;
    Serial.println("loaded dreiviertel");
  }
  else
  {
    *value = false;
    Serial.println("loaded viertel vor");
  }
}


void saveAmbientMin(int newValue)
{
  EEPROM.write((AMBIENTMIN_START), (uint8_t)(newValue));
  EEPROM.write((AMBIENTMIN_START + 1), (uint8_t)(newValue >> 8));
  EEPROM.write((AMBIENTMIN_START + 2), (uint8_t)(newValue >> 16));
  EEPROM.write((AMBIENTMIN_START + 3), (uint8_t)(newValue >> 24));
  EEPROM.commit();
}

void loadAmbientMin(int *value)
{
  int tmp = 0;
  tmp += EEPROM.read(AMBIENTMIN_START);
  tmp += (EEPROM.read(AMBIENTMIN_START + 1) << 8);
  tmp += (EEPROM.read(AMBIENTMIN_START + 2) << 16);
  tmp += (EEPROM.read(AMBIENTMIN_START + 3) << 24);
  *value = tmp;
}

void saveAmbientMax(int newValue)
{
  EEPROM.write((AMBIENTMAX_START), (uint8_t)(newValue));
  EEPROM.write((AMBIENTMAX_START + 1), (uint8_t)(newValue >> 8));
  EEPROM.write((AMBIENTMAX_START + 2), (uint8_t)(newValue >> 16));
  EEPROM.write((AMBIENTMAX_START + 3), (uint8_t)(newValue >> 24));
  EEPROM.commit();
}

void loadAmbientMax(int *value)
{
  int tmp = 0;
  tmp += EEPROM.read(AMBIENTMAX_START);
  tmp += (EEPROM.read(AMBIENTMAX_START + 1) << 8);
  tmp += (EEPROM.read(AMBIENTMAX_START + 2) << 16);
  tmp += (EEPROM.read(AMBIENTMAX_START + 3) << 24);
  *value = tmp;
}

void intitializeDefaultValues()
{
  Serial.println("Loading factory default values");
  saveAmbientMin(1023);
  saveAmbientMax(0);
  saveUtcDiff(1);
  saveSummertimeAuto(true);
  saveDreiviertelMode(false);
  saveBrightnessAuto(true);

  EEPROM.write((FIRSTFLASH_START), (uint8_t)(FLASHPATTERN));
  EEPROM.write((FIRSTFLASH_START + 1), (uint8_t)(FLASHPATTERN >> 8));
  EEPROM.write((FIRSTFLASH_START + 2), (uint8_t)(FLASHPATTERN >> 16));
  EEPROM.write((FIRSTFLASH_START + 3), (uint8_t)(FLASHPATTERN >> 24));
  EEPROM.commit();

  ESP.reset();
}

