#include "UpnpBroadcastResponder.h"
#include "Device.h"
#include <functional>

#define MAX_DEVICES 1

// Multicast declarations
IPAddress ipMulti(239, 255, 255, 250);
const unsigned int portMulti = 1900;
char udpPacketBuffer[512];


UpnpBroadcastResponder::UpnpBroadcastResponder() {
  deviceList = new Device[MAX_DEVICES];
  deviceIndex = 0;
}

bool UpnpBroadcastResponder::start() {
  boolean state = false;

  Serial.println("Starting multicast");

  if (UDP.beginMulticast(WiFi.localIP(), ipMulti, portMulti)) {
    Serial.print("Udp multicast server started at ");
    Serial.print(ipMulti);
    Serial.print(":");
    Serial.println(portMulti);

    state = true;
  }
  else {
    Serial.println("Connection failed");
  }

  return state;
}


void UpnpBroadcastResponder::addDevice(Device& device) {
  String deviceName = device.getAlexaInvokeName();
  Serial.println("Adding " + deviceName + " to UPNP");

  deviceList[deviceIndex] = device;
  deviceIndex++;
}

void UpnpBroadcastResponder::serverLoop() {
  int packetSize = UDP.parsePacket();
  if (packetSize <= 0)
    return;

  IPAddress senderIP = UDP.remoteIP();
  unsigned int senderPort = UDP.remotePort();

  // copy packet into buffer
  UDP.read(udpPacketBuffer, packetSize);

  // check if this is a M-SEARCH for WeMo device
  String request = String((char *)udpPacketBuffer);

  if (request.indexOf('M-SEARCH') > 0)
  {
    if (request.indexOf("urn:Belkin:device:**") > 0)
    {
      Serial.println("UDP Belkin Request..");

      for (int n = 0; n < deviceIndex; n++)
      {
        Device &dev = deviceList[n];

        if (&dev != NULL)
        {
          dev.respondToBelkinSearch(senderIP, senderPort);
          delay(12);
        }
      }
    }
  }
}

