#include "WifiControl.h"
#include "Storage.h"

WifiControl& WifiControl::getInstance()
{
  static WifiControl instance;
  return instance;
}

WifiControl::WifiControl()
{
  connectedToWifi = false;
  ip = IPAddress(192,168,0,10);
}

boolean WifiControl::connectWifi()
{
  char ssid[WIFI_DATA_LENGTH] = "0000000000000000000000000000000";
  char password[WIFI_DATA_LENGTH] = "0000000000000000000000000000000";
  boolean success = true;
  int i = 0;

  loadWifiData(ssid, password);
  Serial.println("Loaded wifi data from storage");
  Serial.print("SSID: ");
  Serial.println(ssid);
  Serial.print("Password: ");
  Serial.println(password);

  WiFi.hostname("Clock");
  WiFi.mode(WIFI_STA);
  WiFi.begin(ssid, password);
  Serial.println("");
  Serial.print("Connecting to WiFi...");

  // Wait for connection
  while (WiFi.status() != WL_CONNECTED)
  {
    delay(500);
    Serial.print(".");
    if (i > 10) {
      success = false;
      break;
    }
    i++;
  }
  if (success)
  {
    ip = WiFi.localIP();
    connectedToWifi = true;
    Serial.println("");
    Serial.print("Connected to ");
    Serial.println(ssid);
    Serial.print("IP address: ");
    Serial.println(ip);
    //sendMail();
  }
  else
  {
    Serial.println("");
    Serial.println("Connection failed.");
    connectedToWifi = false;
    openAccessPoint();
  }
  return success;
}

void WifiControl::openAccessPoint()
{
  Serial.println("Opening AccessPoint.");
  IPAddress subnet(255, 255, 255, 0);

  WiFi.mode(WIFI_AP);
  WiFi.softAPConfig(ip, ip, subnet);
  WiFi.softAP("Clock");
}

boolean WifiControl::isWifiConnected()
{
  return connectedToWifi;
}

IPAddress WifiControl::getCurrentIP()
{
  return ip;
}

void WifiControl::setWifiData(String ssid_string, String pw_string)
{
  char ssid[WIFI_DATA_LENGTH] = "";
  char password[WIFI_DATA_LENGTH] = "";

  strncpy(password, pw_string.c_str(), pw_string.length());
  password[pw_string.length()] = '\0';
  strncpy(ssid, ssid_string.c_str(), ssid_string.length());
  ssid[ssid_string.length() ] = '\0';

  saveWifiData(ssid, password);
}


void WifiControl::hostByName(const char* aHostname, IPAddress& aResult)
{
  WiFi.hostByName(aHostname, aResult);
}

